/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core;

import java.util.List;

import org.hibernate.criterion.Restrictions;

import com.patrikdufresne.managers.AbstractManager;
import com.patrikdufresne.managers.ManagerContext;
import com.patrikdufresne.managers.ManagerException;
import com.patrikdufresne.managers.Query;

/**
 * Object manager for position.
 * 
 * @author Patrik Dufresne
 */
public class PositionManager extends AbstractManager<Position> {

    public PositionManager(PlanimodManagers managers) {
        super(managers);
    }

    @Override
    public Class<Position> objectClass() {
        return Position.class;
    }

    /**
     * List positions for the given Section.
     * 
     * @param section
     *            the section
     * @return list of position
     * @throws ManagerException
     */
    public List<Position> listBySection(final Section section) throws ManagerException {
        return getManagers().query(new Query<List<Position>>() {
            @Override
            public List<Position> run() throws ManagerException {
                return ManagerContext.getDefaultSession().createCriteria(objectClass()).add(Restrictions.eq(Position.SECTION, section)).list();
            }
        });
    }

    /**
     * List classified positions only.
     * 
     * @return list of position
     * @throws ManagerException
     */
    public List<Position> listClassified() throws ManagerException {
        return getManagers().query(new Query<List<Position>>() {
            @Override
            public List<Position> run() throws ManagerException {
                return ManagerContext.getDefaultSession().createCriteria(objectClass()).add(Restrictions.eq(Position.CLASSIFIED, true)).list();
            }
        });
    }

}
