/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core;

import java.util.Map.Entry;

/**
 * 
 * @author Patrik Dufresne
 * 
 */
public class ProductEntry implements Entry<Product, Integer> {

    private Integer count;

    private Product product;

    public ProductEntry(Product product, Integer count) {
        super();
        this.product = product;
        this.count = count;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) return true;
        if (obj == null) return false;
        if (getClass() != obj.getClass()) return false;
        ProductEntry other = (ProductEntry) obj;
        if (count == null) {
            if (other.count != null) return false;
        } else if (!count.equals(other.count)) return false;
        if (product == null) {
            if (other.product != null) return false;
        } else if (!product.equals(other.product)) return false;
        return true;
    }

    @Override
    public Product getKey() {
        return this.product;
    }

    @Override
    public Integer getValue() {
        return this.count;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((count == null) ? 0 : count.hashCode());
        result = prime * result + ((product == null) ? 0 : product.hashCode());
        return result;
    }

    @Override
    public Integer setValue(Integer value) {
        throw new UnsupportedOperationException();
    }

    @Override
    public String toString() {
        return "ProductEntry [product=" + product + ", count=" + count + "]";
    }

}
