/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core;

import java.util.List;

import org.hibernate.criterion.Restrictions;

import com.patrikdufresne.managers.AbstractManager;
import com.patrikdufresne.managers.ManagerContext;
import com.patrikdufresne.managers.ManagerException;
import com.patrikdufresne.managers.Query;

/**
 * Object manager for product.
 * 
 * @author Patrik Dufresne
 */
public class ProductPositionManager extends AbstractManager<ProductPosition> {

    public ProductPositionManager(PlanimodManagers managers) {
        super(managers);
    }

    /**
     * List product-position elements for the given product.
     * 
     * @param product
     *            a product
     * 
     * @return list of product-position
     * @throws ManagerException
     */
    public List<ProductPosition> listByProduct(final Product product) throws ManagerException {
        return getManagers().query(new Query<List<ProductPosition>>() {
            @Override
            public List<ProductPosition> run() throws ManagerException {
                return ManagerContext.getDefaultSession().createCriteria(objectClass()).add(Restrictions.eq(ProductPosition.PRODUCT, product)).list();
            }
        });
    }

    @Override
    public Class<ProductPosition> objectClass() {
        return ProductPosition.class;
    }

}
