/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core.comparators;

import java.util.Calendar;
import java.util.Comparator;

import org.apache.commons.collections.ComparatorUtils;
import org.apache.commons.collections.Transformer;
import org.apache.commons.collections.comparators.ComparatorChain;

import com.planimod.core.Shift;
import com.planimod.core.TimeRange;

public class ShiftComparators {

    /**
     * Compare calendar event by hours (and not date).
     * 
     * @return
     */
    @SuppressWarnings("unchecked")
    public static Comparator<Shift> byHourAndTeam() {
        ComparatorChain chain = new ComparatorChain();

        chain.addComparator(ComparatorUtils.transformedComparator(ComparatorUtils.nullLowComparator(ComparatorUtils.naturalComparator()), new Transformer() {
            @Override
            public Object transform(Object input) {
                Calendar cal = Calendar.getInstance();
                cal.setTime(((TimeRange) input).getStartDate());
                return Integer.valueOf(cal.get(Calendar.HOUR_OF_DAY));
            }
        }));

        chain.addComparator(ComparatorUtils.transformedComparator(ComparatorUtils.nullLowComparator(ComparatorUtils.naturalComparator()), new Transformer() {
            @Override
            public Object transform(Object input) {
                Calendar cal = Calendar.getInstance();
                cal.setTime(((TimeRange) input).getStartDate());
                return Integer.valueOf(cal.get(Calendar.MINUTE));
            }
        }));

        chain.addComparator(ComparatorUtils.transformedComparator(TeamComparators.byName(), new Transformer() {
            @Override
            public Object transform(Object input) {
                return ((Shift) input).getTeam();
            }
        }));

        return chain;
    }

}
