/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core.planif;

import com.planimod.core.Employee;
import com.planimod.core.Team;

/**
 * Used in cases where an employee with preferred seniority has no preferred team.
 * 
 * @author Karl St-Cyr
 * 
 */
public class PreferedSeniorityWithoutPrefTeamCallbackReason implements ICallbackReason {

    private final Employee employee;

    /**
     * 
     * @param employee
     *            the employee with preferred seniority targeted by the callbackReason
     */
    public PreferedSeniorityWithoutPrefTeamCallbackReason(Employee employee) {
        if (employee == null) {
            throw new IllegalArgumentException();
        }
        this.employee = employee;
    }

    public Employee getEmployee() {
        return employee;
    }

    @Override
    public String toString() {
        return "PreferedSeniorityWithoutPrefTeamCallbackReason [employee=" + employee + "]"; //$NON-NLS-1$
    }

}
