/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.ui.theme;

import java.io.IOException;
import java.net.URL;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.SourceDataLine;

public class PlaySound {

    /**
     * Play a "notify" sound.
     */
    public static void playNotify() {
        URL url = PlaySound.class.getResource("notify.wav");
        if (url != null) {
            play(url);
        }
    }

    /**
     * Play the given url file.
     * 
     * @param filename
     *            the name of the file that is going to be played
     */
    public static void play(final URL url) {
        (new Thread() {
            @Override
            public void run() {
                // Open the stream as an audio stream.
                AudioInputStream audioStream;
                SourceDataLine sourceLine;
                try {
                    audioStream = AudioSystem.getAudioInputStream(url);
                    AudioFormat audioFormat = audioStream.getFormat();
                    DataLine.Info info = new DataLine.Info(SourceDataLine.class, audioFormat);
                    sourceLine = (SourceDataLine) AudioSystem.getLine(info);
                    sourceLine.open(audioFormat);
                } catch (Exception e) {
                    return;
                }

                // Read the audio stream.
                final int BUFFER_SIZE = 128000;
                sourceLine.start();
                int nBytesRead = 0;
                byte[] abData = new byte[BUFFER_SIZE];
                while (nBytesRead != -1) {
                    try {
                        nBytesRead = audioStream.read(abData, 0, abData.length);
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                    if (nBytesRead >= 0) {
                        @SuppressWarnings("unused")
                        int nBytesWritten = sourceLine.write(abData, 0, nBytesRead);
                    }
                }
                // Clear the resources.
                sourceLine.drain();
                sourceLine.close();
                try {
                    audioStream.close();
                } catch (IOException e) {
                    return;
                }
            }
        }).start();
    }
}