/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * 
 */
package com.planimod.ui.views.admin.team;

import java.util.Arrays;
import java.util.List;

import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.window.IShellProvider;

import com.patrikdufresne.managers.ManagedObject;
import com.patrikdufresne.managers.jface.AbstractAddAction;
import com.patrikdufresne.util.Localized;
import com.planimod.core.PlanimodManagers;
import com.planimod.core.Team;
import com.planimod.ui.theme.Resources;

/**
 * Action to create a new Team.
 * 
 * @author Patrik Dufresne
 * 
 */
public class AddTeamAction extends AbstractAddAction {
    /**
     * Create a new action.
     * 
     * @param shellProvider
     *            a shell provider in case the action need to display a message
     *            box.
     */
    public AddTeamAction(PlanimodManagers managers, IShellProvider shellProvider, ISelectionProvider selectionProvider) {
        super(managers, shellProvider, selectionProvider);
        setText(Localized.get(AddTeamAction.class, "AddTeamAction.text")); //$NON-NLS-1$
        setToolTipText(Localized.get(AddTeamAction.class, "AddTeamAction.tooltipText")); //$NON-NLS-1$
        setImageDescriptor(Resources.getImageDescriptor(Resources.ICON_LIST_ADD_16));
    }

    /**
     * This implementation always return true.
     * 
     * @see net.ekwos.gymkhana.ui.views.AbstractAddAction#canCreateObject()
     */
    @Override
    protected boolean canCreateObject() {
        return true;
    }

    /**
     * This implementation create a new team.
     * 
     * @see net.ekwos.gymkhana.ui.views.AbstractAddAction#createObjects()
     */
    @Override
    protected List<? extends ManagedObject> createObjects() {
        Team t = new Team();
        t.setName(Localized.get(AddTeamAction.class, "AddTeamAction.newTeamName")); //$NON-NLS-1$
        return Arrays.asList(t);
    }
}