/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.ui.views.employees;

import java.util.Collection;

import com.patrikdufresne.managers.ManagerException;
import com.patrikdufresne.managers.databinding.ManagedObjectComputedSet;
import com.planimod.core.PlanimodManagers;
import com.planimod.core.Position;
import com.planimod.core.PositionManager;

public class ClassifiedPositionComputedSet extends ManagedObjectComputedSet {

    public ClassifiedPositionComputedSet(PlanimodManagers managers) {
        super(managers, Position.class);
    }

    /**
     * This implementation is calling {@link PositionManager#listClassified()}.
     * 
     * @return listof object
     */
    @Override
    protected Collection doList() throws ManagerException {
        return ((PlanimodManagers) getManagers()).getPositionManager().listClassified();
    }

    /**
     * Check if the element should be selected. Default implementation always
     * return true.
     * <p>
     * Subclasses should override this function if it's override
     * {@link #doList()}.
     * 
     * @param element
     *            the element to check
     * @return True if the element should be selected
     */
    @Override
    protected boolean doSelect(Object element) {
        return element instanceof Position && ((Position) element).getClassified();
    }

}
