/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.ui.views.planif;

import java.util.Date;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.ObservablesManager;
import org.eclipse.core.databinding.beans.BeanProperties;
import org.eclipse.core.databinding.observable.value.WritableValue;
import org.eclipse.swt.widgets.Composite;

import com.patrikdufresne.jface.databinding.preference.PreferenceStoreProperties;
import com.patrikdufresne.managers.ManagerException;
import com.patrikdufresne.ui.PageBookViewPart;
import com.patrikdufresne.util.Localized;
import com.planimod.core.PlanimodManagers;
import com.planimod.core.planif.GeneratePlanifContext;
import com.planimod.ui.PlanimodPolicy;
import com.planimod.ui.preference.PlanimodPreferences;

/**
 * Page book to display planif views.
 * 
 * @author Patrik Dufresne
 * 
 */
public class PlanifViewPart extends PageBookViewPart {

    /**
     * The part id.
     */
    public static final String ID = "com.planimod.ui.views.planif.PlanifViewPart"; //$NON-NLS-1$

    /**
     * The generate planif context use by the view parts.
     */
    private GeneratePlanifContext context;

    /**
     * Data binding context.
     */
    private DataBindingContext dbc;
    /**
     * if (this.book != null) { this.book.dispose(); this.book = null; } Used
     * for localization.
     */
    private Localized localized = Localized.load(TaskViewPartPage.class);

    private ObservablesManager om;

    /**
     * Create a new view part.
     */
    public PlanifViewPart() {
        super(ID);
        setTitle(this.localized.get("PlanifViewPart.title")); //$NON-NLS-1$
    }

    /**
     * This implementation creates a view book to hold multiple view part.
     */
    @Override
    public void activate(Composite parent) {

        // Create the context
        try {
            this.context = getSite().getService(PlanimodManagers.class).getTaskManager().createGeneratePlanifContext();
        } catch (ManagerException e) {
            PlanimodPolicy.showException(e);
        }

        super.activate(parent);
        /*
         * Proceed with binding.
         */
        this.dbc = new DataBindingContext();
        this.om = new ObservablesManager();
        this.om.runAndCollect(new Runnable() {
            @Override
            public void run() {
                bindValues();
            }
        });
        this.om.addObservablesFromContext(this.dbc, true, true);
    }

    @Override
    protected void addViews() {
        // Add the views.
        addView(new TaskViewPartPage());
        addView(new ProposedTaskViewPartPage());
    }

    /**
     * This implementation bind the planimod state with the Generate planif
     * context.
     */
    protected void bindValues() {
        /**
         * Bind the context to the date preference.
         */
        if (this.context != null) {
            WritableValue date = new WritableValue(null, Date.class);
            this.dbc.bindValue(date, PreferenceStoreProperties.value(PlanimodPreferences.DATE, String.class).observe(PlanimodPreferences.getPreferenceStore()));
            if (date.getValue() == null) {
                date.setValue(new Date());
            }
            this.dbc.bindValue(BeanProperties.value(GeneratePlanifContext.class, GeneratePlanifContext.WEEK).observe(this.context), date);
        }
    }

    /**
     * This implementation dispose the generate planif context.
     */
    @Override
    public void deactivate() {
        super.deactivate();
        if (this.context != null) {
            this.context.dispose();
            this.context = null;
        }

        /*
         * Dispose the Observable manager
         */
        if (this.om != null) {
            this.om.dispose();
            this.om = null;
        }
        if (this.dbc != null) {
            this.dbc.dispose();
            this.dbc = null;
        }
    }

    @Override
    public void dispose() {
        this.localized = null;
        super.dispose();
    }

    /**
     * Return the generate planif context of this view part. This function
     * should be used by the pages to share the same context.
     * 
     * @return the generate planif context.
     */
    public GeneratePlanifContext getContext() {
        return this.context;
    }

}
