/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core;

import static org.junit.Assert.*;

import java.util.Calendar;
import java.util.Date;

import org.junit.Test;

import com.patrikdufresne.managers.ManagerEvent;
import com.patrikdufresne.managers.ManagerException;

/**
 * This class intend to test the functionnalities provided by the
 * {@link ApplicationSettingManager}.
 * 
 * @author ikus060
 * 
 */
public class ApplicationSettingManagerTest extends ManagerTestCase {

    @Test
    public void testSetByName() {
        addManagerObserver(ManagerEvent.ADD, ApplicationSetting.class);
        try {
            managers.getApplicationSettingManager().setByName("settings_name", "new_value");
        } catch (ManagerException e) {
            fail("Fail to sets the application settings.", e);
        }
        assertEquals("Wrong number of events.", 1, getManagerEvents().size());
        // assertManagerEvent(1, getManagerEvents(), ManagerEvent.ADD, entity);

        try {
            assertEquals("new_value", managers.getApplicationSettingManager().getByName("settings_name"));
        } catch (ManagerException e) {
            fail("Fail to get the application settings.", e);
        }

    }

    @Test
    public void testSetFirstDayOfWeek() throws ManagerException {
        addManagerObserver(ManagerEvent.ADD, ApplicationSetting.class);
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.MILLISECOND, 0);
        managers.getApplicationSettingManager().setFirstDayOfWeek(cal.getFirstDayOfWeek());
        assertEquals("Wrong number of events.", 1, getManagerEvents().size());
        assertEquals(cal.getFirstDayOfWeek(), managers.getApplicationSettingManager().getFirstDayOfWeek());
    }

}
