/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core;

import java.util.Arrays;
import java.util.List;

import org.junit.Assert;
import org.junit.Test;

import com.patrikdufresne.managers.ManagerException;
import com.planimod.core.Section;

/**
 * This test case cover all operation done using the section.
 * 
 * @author patapouf
 * 
 */
public class SectionManagerTest extends ManagerTestCase {
    /**
     * Add the object.
     */
    @Test
    public void AddSection() {
        Section section = new Section();
        try {
            managers.getSectionManager().add(Arrays.asList(section));
        } catch (ManagerException e) {
            fail("Error adding Section", e);
        }
    }

    /**
     * Update the object.
     */
    @Test
    public void UpdateSection() {
        Section section = new Section();
        try {
            managers.getSectionManager().add(Arrays.asList(section));
        } catch (ManagerException e) {
            fail("Error adding object", e);
        }
        section.setName("Section 1");

        try {
            managers.getSectionManager().update(Arrays.asList(section));
        } catch (ManagerException e) {
            fail("Error updating object", e);
        }
    }

    /**
     * Create and then remove the object.
     */
    @Test
    public void RemoveSection() {
        Section section = new Section();
        try {
            managers.getSectionManager().add(Arrays.asList(section));
        } catch (ManagerException e) {
            fail("Error adding object", e);
        }

        int id = section.getId();

        try {
            managers.getSectionManager().remove(Arrays.asList(section));
        } catch (ManagerException e) {
            fail("Error removing object", e);
        }

        try {
            List<Section> list = managers.getSectionManager().list();
            for (Section e : list) {
                if (e.getId() == id) {
                    Assert.fail("Object still exists");
                }
            }
        } catch (ManagerException e) {
            fail("Error listing object", e);
        }

    }
}
